using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PhysParticle : MonoBehaviour
{
    public float mass = 1.0f;

    public bool fixedInPlace = false;
    Vector3 positionFixed;

    // Verlet variables
    Vector3 p_old;
    Vector3 p_now;

    Vector3 gravityForce()
    {
        return Vector3.down * (mass * 0.5f * 9.8f); /* 0.5f to make things fall slower */
    }

    void enforcePositionalConstraint()
    {
        // "please don't sink below ground"
        if (p_now.y < 0.1f) p_now.y = 0.1f;

        // "please stay inside a [-10..+10]^3 box in X and Z"
        p_now.x = Mathf.Clamp(p_now.x, -10.0f, +10.0f);
        p_now.z = Mathf.Clamp(p_now.z, -10.0f, +10.0f);

        // "if you are fixed, then please stay exactly in the starting position"
        if (fixedInPlace) p_now = positionFixed;
    }

    void physStep(float dt)
    {
        p_now = transform.position; // update FROM Unity global position

        // Verlet integration...
        // implicitly: veloclity = (p_now - p_old) / dt;

        Vector3 force = gravityForce(); // only gravity for now
        Vector3 acc = force / mass;
        

        const float DRAG = 0.10f; // let's lose 10% of velocity every sec 
        float lerpFactor = 2 - DRAG * dt; // as an approx of ( 1 + (1-DRAG)^dt )
   
        Vector3 p_next = (1-lerpFactor)*p_old + lerpFactor * p_now 
                         + 
                         acc * (dt * dt);

        p_old = p_now;
        p_now = p_next;

        enforcePositionalConstraint();

        transform.position = p_now;  // update TO Unity global position

    }

    // Start is called before the first frame update
    void Start()
    {
        p_old = transform.position;
        positionFixed = transform.position;
    }

    // FixedUpdate method is called once per (physical) frame
    void FixedUpdate()
    {
        physStep( Time.fixedDeltaTime );
    }
}
